import React, { FC, useState, useEffect } from 'react';
import { Form, FormInstance, notification } from 'antd';
import Image from 'next/image';
import { Social } from '../../../types/social';
import { AUTH_USER } from '../../../constants/common';
import { SocialConfig } from '../../../constants/social';
import { useOauthWindow } from '../../../hooks/useOauthWindow';
import { getAuth } from '../../../actions/auth';
import { getPersonalData } from '../../../actions/profile';
import { CustomInput } from '../../view/CustomInput';
import { CustomInputPassword } from '../../view/CustomInputPassword';
import { FilledButton } from '../../view/FilledButton';
import { OutlinedButton } from '../../view/OutlinedButton';
import { LinkButton } from '../../view/LinkButton';

import { apiClient } from 'lib/apiClient';
import { useGoogleLogin } from '@react-oauth/google';
import { FacebookLoginClient, SuccessResponse } from '@greatsumini/react-facebook-login';
import FacebookLogin from '@greatsumini/react-facebook-login';

import AppleLogin from 'react-apple-login'

const appId = '581373996992475';

type EnterProps = {
    form: FormInstance;
    updateMode: (mode: 'enter' | 'register' | 'reset' | 'finish') => void;
    locale: string;
    handleCancel: () => void;
    updateToken: (token: string) => void;
}

export const EnterContent: FC<EnterProps> = ({
    form,
    updateMode,
    updateToken,
    locale,
    handleCancel
}) => {
    const [isLoading, setIsLoading] = useState<boolean>(false);
    const { openOauthWindow } = useOauthWindow();

    const onLogin = () => {
        form.validateFields().then(() => {
            const { login, password } = form.getFieldsValue();
            setIsLoading(true);
            getAuth(locale, { login, password })
                .then(({ data }) => {
                    if (data.jwtToken) {
                        getPersonalData(locale, data.jwtToken)
                            .then(({ data: profile }) => {
                                localStorage.setItem(AUTH_USER, profile.id.toString());
                                updateToken(data.jwtToken);
                                handleCancel();
                            })
                    }
                })
                .catch((error) => {
                    notification.error({
                        message: 'Error',
                        description: error?.response?.data?.errMessage
                    });
                })
                .finally(() => {
                    setIsLoading(false);
                })
        });
    };


    const googleLogin = useGoogleLogin({
            onError: (err) => {
                console.log(err);
            },        
            onSuccess: async (tokenResponse) => {
                var result = await apiClient.post('http://192.168.0.106:5090/api/auth/tryloginexternal',  { 
                    platform: 0,
                    provider: 4,
                    accesstoken: tokenResponse.access_token
                });
                localStorage.setItem('bbuddy_token_test', result.data.jwtToken);
                console.log(result);
            }
        });

    const facebookLogin = async (response: SuccessResponse) => {
        console.log('Login Success!', response);
        var result = await apiClient.post('http://192.168.0.106:5090/api/auth/tryloginexternal',  { 
            platform: 0,
            provider: 1,
            accesstoken: response.accessToken
        });
        localStorage.setItem('bbuddy_token_test', result.data.jwtToken);
        console.log(result);
    } 
    
    const authRequest = async () => {
        try {
            await apiClient.post('/home/userdata');
            console.log('success');
        }
        catch(err) {
            console.log(err);
        }
    
    }

    const logout = () => {
        localStorage.removeItem('bbuddy_token_test');
    }
    

    const onSocialEnter = (type: Social) => {

        const url = SocialConfig[type].oauthUrl;

        if (!url) return;

        openOauthWindow(url, type, async (event: MessageEvent) => {
            const { data: socialData } = event

            // примерная схема последующей обработки

            // const socialErrors: string[] = [];
            // try {
            //     // отправляем запрос на бэк с данными из соц сети
            //     const { data: { jwtToken } } = await query(socialData);
            //     // обновляем токен
            //     updateToken(jwtToken);
            //     // получаем данные о пользователе
            //     await getAuthUser()
            // } catch (error: any) {
            //     if (error.httpStatus === 449) {
            //         // ошибка, когда отсутствует e-mail
            //
            //         // какие-то дальнейшие действия после получения ошибки, например, закрываем окно и открываем модалку регистрации
            //         handleCancel();
            //         openSocialEmailRequestModal(socialData);
            //     } else if (error.httpStatus === 409) {
            //         // ошибка, когда по переданному email уже существует аккаунт
            //
            //         // какие-то дальнейшие действия после получения ошибки, например, закрываем окно и открываем модалку с вводом пароля
            //         handleCancel();
            //         openSocialPasswordModal(socialData);
            //     } else {
            //         // в остальных случаях записываем ошибку в массив ошибок
            //         socialErrors.push(error.toString());
            //     }
            // }
            //
            // // если все успешно, закрываем окно
            // handleCancel();
        })
    };

    return (
        <>
            <Form form={form} style={{ display: 'flex', gap: 16, flexDirection: 'column' }}>
                <Form.Item
                    name="login"
                    noStyle
                    rules={[
                        {
                            type: 'email',
                            message: 'The input is not valid E-mail'
                        },
                        {
                            required: true,
                            message: 'Please input your E-mail'
                        }
                    ]}
                >
                    <CustomInput
                        size="small"
                        placeholder="E-mail"
                        type="email"
                    />
                </Form.Item>
                <Form.Item
                    name="password"
                    noStyle
                    rules={[{
                        required: true,
                        message: 'Please input your password'
                    }]}
                >
                    <CustomInputPassword
                        size="small"
                        placeholder="Password"
                        autoComplete="off"
                        onPressEnter={onLogin}
                    />
                </Form.Item>
            </Form>
            <FilledButton
                type="primary"
                onClick={onLogin}
                loading={isLoading}
            >
                Enter
            </FilledButton>
            <OutlinedButton onClick={() => updateMode('register')}>Register</OutlinedButton>
            <LinkButton
                type="link"
                onClick={() => updateMode('reset')}
            >
                Forgot password?
            </LinkButton>
            <span>or</span>
            <FacebookLogin 
                initParams={{
                    version: 'v16.0',
                }}
                loginOptions={{
                    return_scopes: true,
                    auth_type: 'rerequest'
                }}
                appId={appId}
                onSuccess={facebookLogin}
                >

            </FacebookLogin>
            <AppleLogin 
                clientId="bbuddy.expert" 
                redirectURI="https://local.apple-sigin.mydomain.com/en/apple-sign-in" 
                responseType='code'
                responseMode = 'query'
            />
            <OutlinedButton
                icon={<Image src="/images/google-logo.png" height={20} width={20} alt="" />}
                onClick={() => googleLogin()}
            >
                Google account
            </OutlinedButton>

            <OutlinedButton                
                onClick={() => authRequest()}
            >
                Test Request
            </OutlinedButton>

            <OutlinedButton                
                onClick={() => logout()}
            >
                Log Out
            </OutlinedButton>


        </>
    );
};
